// -*-c++-*-

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of scatter
//
// scatter is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// scatter is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the
//       Free Software Foundation, Inc.
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#ifndef ZONEMAP_H
#define ZONEMAP_H

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>

#include <Exception/Exception.h>
#include "ScatterDB.h"

#include "zone_info.h"

namespace Scatter {

    class ZoneMap {

    public:
	ZoneMap( const std::string& filename,
		 const int surfno,
		 const int shell,
		 const double z0,
		 const bool in_plane,
		 const bool out_of_plane );
	~ZoneMap( );

	struct Zone {

	    double zmin;
	    double zmax;

	    std::string id;

	    bool in_plane;
	    bool out_of_plane;
	    bool scale_out_of_plane_by_sin_alpha;
	    bool make_out_of_plane_symmetric;

	    typedef boost::shared_ptr<ScatterDB> ScatterDBPtr;

	    ScatterDBPtr db_in_plane;
	    ScatterDBPtr db_out_of_plane;

	    double scale_in_plane;
	    double scale_out_of_plane;

	    Zone( const ZoneInfo& zi );

	    struct cmp {

		bool operator() ( const Zone* const& z1, double const& zmin ) {

		    return z1->zmin < zmin;
		}

		bool operator() ( const Zone* const& z1, const Zone* const& z2 ) {

		    return z1->zmin < z2->zmin;
		}

	    };

	};

	Zone *findZone( double z );

	double zmin( ) { return ( *_zones.begin() )->zmin ; }
	double zmax( ) { return ( *(_zones.end()-1) )->zmax ; }

    private:

	typedef std::vector<Zone*> Zones;

	Zones _zones;

    };

}
#endif // ! ZONEMAP_H
